#!/usr/bin/env bash
#!/bin/env bash

## Name: GFFprof.sh
## Version: 1.2.2
## Date: Mar 11, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Create annotation from RNAseq data


ECHO_N="echo -n"
ECHO_E="echo -e"
PREFIX="."
MAXCPU=1
GFF=""

usage() {
    $ECHO_E "Usage: ./GFFprof.sh [OPTIONS]\n"
    $ECHO_E "Optional:"
    $ECHO_E "-C|--CPU MAXCPU"
    $ECHO_E "\tRun up to MAXCPU processes in parallel. By default, 1."
    $ECHO_E "-g|--gff GFF"
    $ECHO_E "\tUse GFF file for extension. By default, no given GFF is required."
    $ECHO_E "-h|-?|--help"
    $ECHO_E "\tPrint this help."
    $ECHO_E "-p|--prefix PREFIX"
    $ECHO_E "\tThis is the same directory as PREFIX given in configuration step. All generated scripts are located in PREFIX/script. By default, PREFIX=.\n"
    
    exit 1
}

if [ ! $1 ]
then
    usage
fi

while :
do
    case $1 in
	-h | --help | -\?)
	    usage
	    ;;
	-g | --gff)
	    GFF=$2
	    shift 2
	    ;;
	--gff=*)
            GFF=${1#*=} #delete everything up till '='
            shift
            ;;
	-p | --prefix)
	    PREFIX=$2
	    shift 2
	    ;;
	--prefix=*)
            PREFIX=${1#*=} #delete everything up till '='
            shift
            ;;
	-C | --CPU)
	    MAXCPU=$2
	    shift 2
	    ;;
	--CPU=*)
            MAXCPU=${1#*=}
            shift
            ;;
	-*)
	    $ECHO_ "Unknown option: $1" >&2
	    shift
	    ;;
	--) #end of all options
	    shift
	    break
	    ;;
	*) #no more options, stop while loop
	    break
	    ;;
    esac
done

## absolute prefix
cd $PREFIX
PREFIX=$PWD
cd - > /dev/null

## directory containing sources
SRC=$0
SRC=`dirname $SRC`
cd $SRC
SRC=$PWD
cd - > /dev/null

$ECHO_E "\nGFFPROF..."

rm -f $PREFIX/rnaseq/*refined*sam

## run in parallel init_*
find $PREFIX/script -name GFFprof_\* | xargs -P $MAXCPU -I {} bash -c "bash {}"

if [ $? -ne 0 ]
then
    echo "Errors while running $PREFIX/script/GFFprof*"
    exit 1
fi

## create filename list
if [ $GFF ]
then
    GFF=`basename $GFF`    
    $ECHO_E "$PREFIX/genome/NEG_tsc_fusion_$GFF" > $PREFIX/rnaseq/tscfile.list
    $ECHO_E "$PREFIX/genome/POS_tsc_fusion_$GFF" >> $PREFIX/rnaseq/tscfile.list
else
    $ECHO_N > $PREFIX/rnaseq/new_annotation.gff
    $ECHO_E "$PREFIX/rnaseq/new_annotation.gff" > $PREFIX/rnaseq/tscfile.list
    $ECHO_E "$PREFIX/rnaseq/new_annotation.gff" >> $PREFIX/rnaseq/tscfile.list
fi

$ECHO_E "$PREFIX/rnaseq/NEG_tsc.gff" >> $PREFIX/rnaseq/tscfile.list
$ECHO_E "$PREFIX/rnaseq/POS_tsc.gff" >> $PREFIX/rnaseq/tscfile.list
$ECHO_E "$PREFIX/rnaseq/UNS_tsc.gff" >> $PREFIX/rnaseq/tscfile.list
$ECHO_E "Done: create filename list"

cd $PREFIX/rnaseq

## discover non-annotated transcripts and extend annotated transcripts: (*_rnaseq.gff)
$SRC/perl/runtsccompare.pl -i $PREFIX/rnaseq/tscfile.list --genome $PREFIX/genome/genomesize.txt --src $SRC
$ECHO_E "Done: discover non-annotated transcripts and extend annotated transcripts"

## join all transcripts from rnaseq
cat $PREFIX/rnaseq/NEG_rnaseq.gff $PREFIX/rnaseq/POS_rnaseq.gff $PREFIX/rnaseq/UNS_rnaseq.gff | sort -k1,1 -k4,4n -k5,5n > $PREFIX/rnaseq/prof_annotation_raw.gff
$ECHO_E "Done: join all transcripts from rnaseq"

## recover exon features
$SRC/perl/jointscexon.pl -i $PREFIX/rnaseq/prof_annotation_raw.gff -o $PREFIX/rnaseq/prof_annotation.gff
rm $PREFIX/rnaseq/exons.tsv $PREFIX/rnaseq/transcripts.gff
$ECHO_E "Done: recover exon features"

## separate transcripts and exons for each chromosome
while read line
do
    CHRARR=(${line//"\t"/ })
    echo "awk -v arg=${CHRARR[0]} '\$1==arg' $PREFIX/rnaseq/prof_annotation_raw.gff > $PREFIX/rnaseq/gff/prof_annotation_raw_${CHRARR[0]}.gff" > $PREFIX/script/gffprof_${CHRARR[0]}.sh
    echo "$SRC/perl/jointscexon.pl -i $PREFIX/rnaseq/gff/prof_annotation_raw_${CHRARR[0]}.gff -e $PREFIX/rnaseq/exon/prof_exon_${CHRARR[0]}.tsv -t $PREFIX/rnaseq/gff/prof_annotation_noexon_${CHRARR[0]}.gff" >> $PREFIX/script/gffprof_${CHRARR[0]}.sh
    echo "awk -v arg=- '\$7==arg' $PREFIX/rnaseq/gff/prof_annotation_noexon_${CHRARR[0]}.gff > $PREFIX/rnaseq/gff/prof_NEG_noexon_${CHRARR[0]}.gff" >> $PREFIX/script/gffprof_${CHRARR[0]}.sh
    echo "awk -v arg=+ '\$7==arg' $PREFIX/rnaseq/gff/prof_annotation_noexon_${CHRARR[0]}.gff > $PREFIX/rnaseq/gff/prof_POS_noexon_${CHRARR[0]}.gff" >> $PREFIX/script/gffprof_${CHRARR[0]}.sh
    echo "awk -v arg=. '\$7==arg' $PREFIX/rnaseq/gff/prof_annotation_noexon_${CHRARR[0]}.gff > $PREFIX/rnaseq/gff/prof_UNS_noexon_${CHRARR[0]}.gff" >> $PREFIX/script/gffprof_${CHRARR[0]}.sh
done < $PREFIX/genome/genomesize.txt

## run in parallel gffprof_*
ls $PREFIX/script/gffprof_*.sh | xargs -P $MAXCPU -I {} bash -c "bash {}";

rm -rf $PREFIX/script/gffprof_*.sh
rm -rf $PREFIX/rnaseq/exons.tsv $PREFIX/rnaseq/transcripts.gff

$ECHO_E "Done: separate transcripts and exons for each chromosome"
